import socket
import asynchat
import asyncore
import threading
import json
import traceback
import sys
import logging
import time

"""
TODO:
- AsyncRepeater umi:
    - poslat pozadavek
    - dat posledni odpoved z fronty, odpoved i s pozadavakem?
    - restartovat connect
    - restartovt listen (zmena portu?)

    - odpovedi automaticky posila dale, pokud neni spojeni, zahazuje
    - pozadavky bud uklada do sve fronty, nebo preposila (podle adresy), pokud neni spojeni,
      vraci odpoved s chybou

Struktura zprav:
zadani pozadavku : recipient, data
odeslani pozadavku: id, recipient, sender, data; (data also stored)
odpoved: id,  sender, answer data
vraceni odpovedi: id, recipient, answer, request
"""


_terminator = '\n'.encode()

def _pack_message(id, sender, recipient, data):
    str_json=json.dumps((id, sender, recipient, data))
    return str_json.encode() + _terminator

def _unpack_message(msg):
    return json.loads(msg.decode())

def _exception_answer(e):
    """
    Retrun answer with catched exception, stack, ...
    :param e: The excpetion.
    :return:
    """
    return { 'error': 'Exception', 'exception': repr(e), 'traceback': traceback.format_tb() }

# def _close_request():
#     """
#     Auxiliary request to inform service about clossed server-side connection.
#     Action should return no answer.
#     :return:
#     """
#     return {'action': 'request_close'}

# def _close_answer():
#     """
#     Auxiliary answer to inform service about clossed clinet-side connection.
#     :return:
#     """
#     return {'action': 'on_answer_close'}

class RequestData:
    """
    Format of request returned by the AsyncRepeater.
    The processing of the request should not depend on id and sender, but
    they are provided for possible logging.

    Attributes:
        self.id          request id, unique per child service
        self.sender      address of the service the send the request
        self.request     request itself
    """
    def __init__(self, id, sender, data):
        self.id = id            # request id, unique per child service
        self.sender = sender    # address of the service the send the request
        self.request = data     # request itself

class AnswerData:
    """
    Format of answer returned by the AsyncRepeater.

    Attributes:
        self.id         request id, unique id= id
        self.sender     sender of answer, target of request
        self.request    request_data
        self.answer     answer_data
        self.on_answer  data stored when request was sent
    """
    def __init__(self, id, sender, request, answer, on_answer):
        self.id = id
        self.sender = sender # target of the request
        self.request = request
        self.answer = answer
        self.on_answer = on_answer



class _ClientDispatcher(asynchat.async_chat):
    """
    Client part of the communication. Send requests, process answers.

    The _ClientDispatcher have following states:

        1. Waiting to get address from child through StarterServer.

        2. Waiting for call connect_to_address(). Have valid self._remote_address, but not connected yet (can not send or recieve message).

        3. Connecting.

        4. Connected.
    """
    def __init__(self, repeater_address, server_dispatcher, connection, get_answer_on_connect=None):
        """
        :param repeater_address: Address of this repeater.
        :param server_dispatcher: Server side of the repeater (can be None). Used to resend answers.
        :param connection: A conncetion object for port forwarding of final connection to the child repeater.
        """
        self.repeater_address = repeater_address
        # Own address given by parent repeater.
        self.address = None
        # Local address to which we connect for permanent connection.
        self._remote_address = None
        # Romete address at which child is listening.
        # (ip, port) on remote machine we connect to over tunnel
        self.server = server_dispatcher
        self.answers = []
        # Received answers for local service.
        self.sent_requests={}
        # Sent requests from local service
        self.sent_requests_lock = threading.Lock()
        """Lock for self.sent_requests"""
        self.request_id=0
        # ID of the next request.
        self.received_data = bytearray()
        # Storage for partially received answer.
        self.connection = connection
        """connection for creating tunnels"""
        self.forwarded_local_port = None
        """forwarded local port for connection to service"""
        self.forwarded_local_port_id = None
        """forwarded local port connection id"""
        self.forwarded_remote_port = None
        """forwarded remote port for back connection from service"""
        self.forwarded_remote_port_id = None
        """forwarded remote port connection id"""

        self.connect_to_address_queue = []
        """Queue for connect request"""
        self.push_queue = []
        """Queue for push request"""
        self.request_close = False
        """True if service thread wants to close dispatcher."""
        self.request_close_done = False
        """True if dispatcher was closed."""

        self.set_remote_address_lock = threading.Lock()
        """Lock for set_remote_address()"""

        asynchat.async_chat.__init__(self)

    def connect_to_address_safe(self, address):
        """
        Thread safe version of connect_to_address.
        :param address:
        :return:
        """
        self.connect_to_address_queue.append(address)

    def connect_to_address(self, address):
        """
        Connect dispatcher to child service.
        :param address: address where child service listening
        :return:
        """
        self.close()
        #self.discard_buffers()
        self.received_data.clear()

        # clear push_queue
        for i in range(len(self.push_queue)):
            self.push_queue.pop(0)

        # create request 0
        with self.sent_requests_lock:
            self.sent_requests[0] = (None, {"action": "on_answer_connect", "data": None})

        self.address = address
        self.create_socket()
        self.socket.settimeout(1)
        logging.info("Connecting: %s" % (str(self.address)))
        try:
            self.connect(self.address)
        except OSError:
            pass

    # def is_connected(self):
    #     return self.connected

    def get_answers(self):
        """
        Extract and return (almost) all recieved requests to proceess.
        Since this extraction is done by the single thread, it should be safe.
        :return:
        """
        copy=[]
        size=len(self.answers)
        for i in range(size):
            answer = self.answers.pop(0)
            logging.info("Copy answer: " + str(answer))
            (id, sender, reciever, answer_dict) = answer
            with self.sent_requests_lock:
                if id not in self.sent_requests:
                    continue
                (request, on_answer) = self.sent_requests.pop(id)
            copy.append( AnswerData(id, sender, request, answer_dict, on_answer) )
        return copy


    def send_request(self, target, data, on_answer):
        """
        Send request down through the connection tree.
        :param target: repeter list, continuous sublist of [ backend_id, mj_id, job_id]
        :param data: JSON data
        :return:
        """
        self.request_id +=1
        id = self.request_id
        with self.sent_requests_lock:
            self.sent_requests[id] = (data, on_answer)
        if self.connected:
            logging.info('Push: %s'%( _pack_message(id, self.repeater_address, target, data)) )
            self.push_safe( _pack_message(id, self.repeater_address, target, data) )
        else:
            self.answers.append((id, None, None, {"error": None}))

    def set_remote_address(self, address):
        # Set remote address of the child repeater.
        # This force port forwarding through connection object and connect.
        with self.set_remote_address_lock:
            if self._remote_address is not None:
                return

            self._remote_address = address

        # forward tunnel
        self.forwarded_local_port_id = self.connection._id
        self.forwarded_local_port = self.connection.forward_local_port(self._remote_address[1])

        #self.connect_to_address(("localhost", local_port))
        self.connect_to_address_safe((self._remote_address[0], self.forwarded_local_port))

    def reconnect(self):
        """
        Reconnect dispatcher.
        :return:
        """
        if self.connection._id > self.forwarded_local_port_id:
            self.forwarded_local_port_id = self.connection._id
            self.forwarded_local_port = self.connection.forward_local_port(self._remote_address[1])
        self.connect_to_address_safe((self._remote_address[0], self.forwarded_local_port))

    def forward_remote_port(self, local_port):
        """
        Forwards remote port for service back connection.
        :param local_port:
        :return:
        """
        self.forwarded_remote_port_id = self.connection._id
        self.forwarded_remote_port = self.connection.forward_remote_port(local_port)
        return self.forwarded_remote_port

    def close_forwarded_ports(self):
        """
        Close forwarded ports opened by dispatcher.
        :return:
        """
        # forwarded_local_port
        if (self.forwarded_local_port_id is not None) and (self.forwarded_local_port_id == self.connection._id):
            self.connection.close_forwarded_local_port(self.forwarded_local_port)

        # forwarded_remote_port
        if (self.forwarded_remote_port_id is not None) and (self.forwarded_remote_port_id == self.connection._id):
            self.connection.close_forwarded_remote_port(self.forwarded_remote_port)

    def process_queues(self):
        """
        Process requests from other threads.
        :return:
        """
        if self.request_close:
            if not self.request_close_done:
                self.close()
                self.request_close_done = True
        else:
            # connect to address
            l = len(self.connect_to_address_queue)
            if l > 0:
                address = None
                for i in range(l):
                    address = self.connect_to_address_queue.pop(0)
                self.connect_to_address(address)

            # push
            l = len(self.push_queue)
            if l > 0:
                for i in range(l):
                    self.push(self.push_queue.pop(0))

    def push_safe(self, data):
        """
        Thread safe version of push.
        :param data:
        :return:
        """
        self.push_queue.append(data)


    """
    Remaining are implementations of asyncore methods.
    """

    def handle_connect(self):
        logging.info("Connected")
        self.set_terminator(_terminator)

        # create answer 0
        self.answers.append((0, None, None, {"data": None}))

    def handle_close(self):
        logging.info("handle_close")
        self.close()

    def handle_error(self):
        self.close()

        with self.sent_requests_lock:
            for k in self.sent_requests.keys():
                self.answers.append((k, None, None, {"error": "connection"}))

        # try:
        #     raise
        # except ConnectionRefusedError:
        #     print("ConnectionRefusedError")
        # except:
        #     logging.error("Uncatch exception in repeater.run thread:\n" + "".join(traceback.format_exception(*sys.exc_info())))

    def collect_incoming_data(self, data):
        """Read an incoming message from the client and put it into our outgoing queue."""
        logging.info("Collect")
        self.received_data.extend(data)

    def found_terminator(self):

        msg = _unpack_message(self.received_data)
        logging.info("Client, message: "+ str(msg))
        self.received_data = bytearray()
        if msg:
            recipient = msg[2]
            logging.info("recp: %s addr: %s"%(str(recipient), str(self.repeater_address)))
            if recipient != self.repeater_address:
                assert self.server, "Wrong recipient: %s"%(str(recipient))
                # forward answer
                self.server.push(msg)
            else:
                # process answers t own reqests
                self.answers.append( msg )




class Server(asyncore.dispatcher):
    def __init__(self,  repeater, port, clients):
        """
        host - get automatically
        :param port - port ( same as in socket module)
        """
        self.repeater = repeater
        asyncore.dispatcher.__init__(self)
        self.server_dispatcher = ServerDispatcher(repeater.repeater_address, port, clients)

        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.bind( ("", port) )
        self.address=self.socket.getsockname()
        self.answers=[]
        self.listen(5)


    def get_dispatcher(self):
        return self.server_dispatcher

    def handle_accept(self):
        # Called when a client connects to our socket

        # stop starter client
        self.repeater._starter_client_attempting = False

        # close previous connection
        #self.server_dispatcher.close()
        #self.server_dispatcher.discard_buffers()
        self.server_dispatcher.received_data.clear()

        client_info = self.accept()
        logging.info("Incomming connection accepted.")
        #print("Incomming connection accepted.")
        self.server_dispatcher.accept(client_info[0])
        return

    def handle_close(self):
        self.close()

class ServerDispatcher(asynchat.async_chat):
    def __init__(self,  repeater_address, port, clients):
        """
        host - get automatically
        :param port - port ( same as in socket module)
        """
        self.repeater_address = repeater_address
        logging.info("Raddr: %s"%(str(repeater_address)))
        # Own address given by parent repeater.
        self.port = port
        # Port we will connect after accept.
        self.clients = clients
        # Dict of client dispatchers.
        self.requests = []
        # Recieved requests to be processed.
        self.request_senders={}
        # Dict  id-> sender. Used to send answers to correct origin.
        self.received_data = bytearray()
        # Uncomplete request.
        self.answer_id = 0
        # Server numbering of answers.

        self.push_queue = []
        """Queue for push request"""


    def accept(self, socket):
        logging.info("Accept")
        #socket.settimeout(2)
        asynchat.async_chat.__init__(self, sock = socket)
        #self.set_socket(socket)
        self.set_terminator(_terminator)
        return

    def get_requests(self):
        """
        Extract and return (almost) all recieved requests to proceess.
        Since this extraction is done by the single thread, it should be safe.
        :return:
        """
        copy=[]
        req_len=len(self.requests)
        #logging.info("GET requests len: %d"%( len(self.requests) ))
        for i in range(req_len):
            request = self.requests.pop(0)
            logging.info("copy req: " + str(request) )
            (request_id, sender, recipient, data) = request
            self.request_senders[self.answer_id]=(request_id, sender)
            copy.append( RequestData(self.answer_id, sender, data) )
            self.answer_id += 1
        return copy

    def send_answer(self, answer_id, data):

        if answer_id in self.request_senders:
            (id, sender) = self.request_senders.pop(answer_id)
            if self.connected:
                logging.info("send answer: " + str(_pack_message(id, self.repeater_address, sender, data)) )
                self.push_safe( _pack_message(id, self.repeater_address, sender, data) )

    def process_queues(self):
        """
        Process requests from other threads.
        :return:
        """
        # push
        l = len(self.push_queue)
        if l > 0:
            for i in range(l):
                self.push(self.push_queue.pop(0))

    def push_safe(self, data):
        """
        Thread safe version of push.
        :param data:
        :return:
        """
        self.push_queue.append(data)


    """
    Remaining are implementations of asyncore methods.
    """
    def collect_incoming_data(self, data):
        """
        Read an incoming message from the client and put it into our outgoing queue.
        """
        self.received_data.extend(data)

    def found_terminator(self):
        """
        The end of a command or message has been seen.
        """
        msg = _unpack_message(self.received_data)
        logging.info("Server, message: " + str(msg))
        self.received_data = bytearray()
        if msg:
            (id, sender, recipient, request) = msg
            if len(recipient) == 0:
                # empty recipient, we have to process
                logging.info("process: " + str(msg))
                self.requests.append( msg )
                logging.info("requests len: %d"%( len(self.requests)  ))
            else:
                try:
                    client = self.clients[recipient[0]]
                except KeyError:
                    msg={ 'error' : "Unknown recipient", 'recipient': self.own_address + recipient[0] }
                    self.push( _pack_message(id, self.repeater_address, sender, msg ) )
                else:
                    if client.connected:
                        try:
                            # forward message
                            client.push( _pack_message(id, sender, recipient[1:], request) )
                        except Exception as e:
                            self.push( _pack_message(id, self.repeater_address, sender, _exception_answer(e)) )
                            return

                    else:
                        msg = {'error': "Recipient not connected", 'recipient': self.repeater_address + recipient[0]}
                        self.push(_pack_message(id, self.repeater_address, sender, msg) )

    def handle_close(self):
        #self.logger.debug('handle_close()')
        self.close()
        return

    def handle_error(self):
        self.close()


class StarterServer(asyncore.dispatcher):
    """
    Server which accepts reverse connection from child repeaters.
    """
    def __init__(self, async_repeater):
        """
        :param async_repeater:
        """
        asyncore.dispatcher.__init__(self)

        self.async_repeater = async_repeater

        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.bind(("", 0))
        self.address=self.socket.getsockname()
        self.listen(5)
        logging.info("Starter Server listening at address: {}".format(self.address))

    def handle_accepted(self, sock, addr):
        StarterServerDispatcher(sock, self.async_repeater)
        #print(sock.getsockname())


class StarterServerDispatcher(asyncore.dispatcher):
    def __init__(self, sock, async_repeater):
        super().__init__(sock)
        self.async_repeater = async_repeater

    def handle_read(self):
        peername = self.socket.getpeername()
        data = self.recv(1024)
        #print(data)
        if data:
            s = data.decode().split("\n", maxsplit=2)
            # Skip connections with wrong number of parameters.
            if len(s) == 2:
                child_id = int(s[0])
                port = int(s[1])
                #print((child_id, port))
                #print(self.async_repeater.clients)

                try:
                    client = self.async_repeater.clients[child_id]
                except KeyError:
                    pass
                else:
                    if not client.request_close:
                        client.set_remote_address((peername[0], port))
                        logging.info("Initial back connection done.")
        self.close()
        # We close also if we get wrong data. As whole connection is from bad guy.

    def handle_error(self):
        self.close()


class AsyncRepeater():
    """
    Interface to request-answer communication tree.

    AsyncRepeaters form a tree with root in client, first level with single node (backend),
    second level corresponding to multi jobs, and third level corresponding to Jobs.

    Repeaters handle packing and passing a request from any node to its particular child
    and then propagating back the answer to the request.
    Repeater do not process requests itself.
    Only in the case of error it sends the error answer itself.
    """
    def __init__(self, repeater_address, parent_address=("", 0), max_client_id=0):
        """
        :param repeater_address: Repeater address as a list of IDs for path from root repeater to self.
            last item is ID of self repeater. Empty list mean root repeater.
        :param listen_port: The port for permanent connection of the parent repeater to the self.
               None - means no listening port is open (i.e. root repeater)
               0 - get by kernel (usual case)
        :param parent_address:
                socket address ( address, port) of the parent repeater to connect for initialization.
        """
        self.repeater_address = repeater_address
        self.parent_address = parent_address
        self.max_client_id = max_client_id
        self.clients = {}
        """ Dict of clients. Keys client_id. """
        self.clients_lock = threading.Lock()
        """Lock for clients"""
        self._server = None
        self._server_dispatcher = None
        self.listen_port = None

        self._starter_client_thread = None
        self._starter_client_attempting = False
        if self.parent_address[0] != "":
            self._server = Server(self, 0, self.clients)
            self.listen_port = self._server.address[1]
            self._server_dispatcher = self._server.get_dispatcher()

            self._starter_client_attempting = True

        self._starter_server = StarterServer(self)
        self._loop_thread = None
        self._loop_closing = False

    def run(self):
        """
        Start the repeater loop in separate thread.
        Start starter client.
        :return: None
        """
        self._loop_thread = threading.Thread(target=self._loop, daemon=True)
        self._loop_thread.start()
        self._loop_closing = False
        logging.info("Repeater loop started.")

        if self._starter_client_attempting:
            self._starter_client_thread = threading.Thread(target=self._starter_client_run, daemon=True)
            self._starter_client_thread.start()
            logging.info("Starter client started.")

    def _loop(self):
        """
        Repeater loop.
        :return:
        """
        while not self._loop_closing:
            asyncore.loop(timeout=0.01, count=1)
            self._process_queues()

    def _process_queues(self):
        """
        Process requests from other threads.
        :return:
        """
        # server dispatcher
        if self._server_dispatcher is not None:
            self._server_dispatcher.process_queues()

        # clients
        with self.clients_lock:
            clients = list(self.clients.values())
        for c in clients:
            c.process_queues()


    def add_child(self, connection, remote_address=None, id=None):
        """
        Setup connection to a child repeater.
        1. Create a Dispatcher for connection to the child repeater, but do not connect.
        2. Ask `connection` for remote port forwarding to the StarterServer port.
        2. StarterServer wait for back connection from the child repeater, to get its listening port.
        3. Then StarterServer asks the Dispatcher to connect to the child repeater through the `connection`
         passed as parameter (local port forwarding).

        :param connection: The connection object used for port forwarding (None forwarding or SSH).
        :param remote_port: If the remote port is known, we skip the StarterServer connection and call
        connection to the child repeater directly.

        :return: ( child_id, forwarded_remote_port)
            child_id: ID of the child repeater it must be passed to the repeater when its process is started
            (e.g. as command parameter)
            forwarded_remote_port: a port on the remote machine the child repeater runs on and  to which it should connect
            to send its listening_port to the parent.

        TODO: test and possibly finish support for the remote_port parameter.
        IDEA: to check that we get correct child we may generate child ids not incermentaly, but at random.
             Then it servers also as a unique token to check that the correct repeater is connecting to the StarterServer.
             Must keep generating of ID atomic.
        """
        client = _ClientDispatcher(self.repeater_address, self._server_dispatcher, connection)

        with self.clients_lock:
            if id is not None:
                assert id not in self.clients
                if id > self.max_client_id:
                    self.max_client_id = id
            else:
                self.max_client_id += 1
                id = self.max_client_id

            self.clients[id] = client

        if remote_address:
            client.set_remote_address(remote_address)
            remote_port = None
        else:
            # remote tunnel
            local_port = self._starter_server.address[1]
            remote_port = client.forward_remote_port(local_port)

        return id, remote_port

    def reconnect_child(self, id):
        """
        Reconnect client dispatcher.
        :param id:
        :return:
        """
        self.clients[id].reconnect()

    def remove_child(self, id):
        """
        Remove connection to child repeater.
        :param id:
        :return:
        """
        self.clients[id].request_close = True

    def discard_closed_childs(self):
        """
        Remove closed childs from self.clients and close their forwarded ports.
        :return:
        """
        discard = []
        with self.clients_lock:
            for k in list(self.clients.keys()):
                if self.clients[k].request_close_done:
                    discard.append(self.clients.pop(k))
        for d in discard:
            d.close_forwarded_ports()


    # def close_child_repeater(self, id):
    #     """
    #     Check that socket to child is closed. Delete the dispatcher.
    #
    #     TODO: thread safe?
    #     :param id:
    #     :return:
    #     """
    #     self.clients[id].close()
    #     del self.clients[id]
    #     return

    def send_request(self, target, data, on_answer):
        """
        Send request down through the connection.

         The request is given by 'data'., which could be any JSON serializable structure.
         The request is sent to the 'target' service.
         The on_naswer parameter gives additional data that are returned together with the answer to the request.

        TODO: test and finish.
        :param target: target repeater address relative to local repeater.
            I.e. sublist of [ backend_id (=0), mj_id, job_id ]
        :param data: any JSON serializable structure
        :param on_answer: data returned together with answer
        :return: None
        """
        #assert target, "Empty target."
        self.clients[target[0]].send_request(target[1:], data, on_answer)


    def send_answer(self, id, data):
        """
        Send an answer from local service. This is automatically paired with
        the corresponding request. From which we get the target of the answer.
        :param id: Id of request to which we answer.
        :param data: Answer data.
        :return:
        """
        self._server_dispatcher.send_answer(id, data)

    def get_requests(self):
        """
        Get list of requests to process.
        :return: [ RequestData, ... ]
        """
        #logging.info("GR")
        if self._server_dispatcher is not None:
            return self._server_dispatcher.get_requests()
        else:
            return []

    def get_answers(self, child_id):
        """
        Get list of answers from client (child_id).
        :return: [ AnswerData, ... ]
        """
        #logging.info("GA")
        return self.clients[child_id].get_answers()


    def close(self):
        """
        Close repeater.
        :return:
        """
        # repeater loop
        if self._loop_thread is not None:
            self._loop_closing = True
            self._loop_thread.join(timeout=1)
            if self._loop_thread.is_alive():
                logging.warning("Repeater loop closing timeout.")

        # server
        if self._server is not None:
            self._server.close()

        # server dispatcher
        if self._server_dispatcher is not None:
            self._server_dispatcher.close()

        # client dispatchers
        for c in self.clients.values():
            if c is not None:
                c.close()
                c.close_forwarded_ports()

        # starter server
        self._starter_server.close()

    def _starter_client_run(self):
        logging.info("Attempting for back to parent initial connection to address: {}".format(self.parent_address))
        while self._starter_client_attempting:
            s = socket.socket()
            try:
                s.connect(self.parent_address)
                data = "{}\n{}".format(self.repeater_address[-1], self.listen_port).encode()
                s.sendall(data)
            except ConnectionRefusedError:
                pass
            except OSError:
                pass
            finally:
                s.close()
            time.sleep(10)
